/*****************************************************************************************************************************
--
--    File Name    : Image_Edge_Detection_tb.v 

--    Description  : This module provides the test environment for Image Edge Detection IP.
						
					 For testing this IP, inputs should come from RGB to YCbCr IP and then outputs should be 
					 connected to YCbCr to RGB IP.
					 
--                   The following file is required for the simulation :
					 RGB_in.txt -->  It contains hex values which are in the following format: 
									 First 2 bytes width of the image, next 2 bytes height of the image.
									 Then follows B,G and R values.
									 For more details visit Microsemi Video Web Page. 

-- Targeted device : Microsemi-SoC                     
-- Author          : India Solutions Team

-- SVN Revision Information:
-- SVN $Revision: TBD
-- SVN $Date: TBD
--
--
--
-- COPYRIGHT 2016 BY MICROSEMI 
-- THE INFORMATION CONTAINED IN THIS DOCUMENT IS SUBJECT TO LICENSING RESTRICTIONS 
-- FROM MICROSEMI CORP.  IF YOU ARE NOT IN POSSESSION OF WRITTEN AUTHORIZATION FROM 
-- MICROSEMI FOR USE OF THIS FILE, THEN THE FILE SHOULD BE IMMEDIATELY DESTROYED AND 
-- NO BACK-UP OF THE FILE SHOULD BE MADE. 
-- 

--******************************************************************************************************************************/
`timescale 1ns/1ps
module edge_detection_test;
  /************************************************************************
          Parameters 
  *************************************************************************/
  parameter HEIGHT = 200;
  parameter WIDTH = 320;
  parameter CLKPERIOD = 10;
  parameter WAIT = 0;
  parameter DATA_BIT_WIDTH = 24; //pixel width
  parameter IMAGE_FILE_NAME = "RGB_in.txt";  
  //Parameters from the DUT
  parameter g_RGB_DATA_BIT_WIDTH = 8;
  parameter g_YCbCr_DATA_BIT_WIDTH = 8;
  parameter g_DATAWIDTH = 8;
  parameter g_X_RES_WIDTH = 11;
  localparam MAX_SIZE = (HEIGHT * WIDTH * (DATA_BIT_WIDTH / g_DATAWIDTH)) + 4;
  localparam MAX_SIZE_24 = (HEIGHT * WIDTH) + 1;
  /************************************************************************
          Inputs and Outputs to the DUT 
  *************************************************************************/
  reg Clock;
  reg Reset;
  reg[g_DATAWIDTH - 1 : 0] Red;
  reg[g_DATAWIDTH - 1 : 0] Green;
  reg[g_DATAWIDTH - 1 : 0] Blue;
  reg dataValidIn_rgb;
  reg H_sync_in;
  reg V_sync_in;
  reg H_active_in;
  reg V_active_in;
  wire[g_DATAWIDTH - 1 : 0] Y_out;
  wire[g_DATAWIDTH - 1 : 0] Cb_out;
  wire[g_DATAWIDTH - 1 : 0] Cr_out;
  wire dataValidOut_rgb;
  wire H_sync_out;
  wire V_sync_out;
  wire H_active_out;
  wire V_active_out;
  reg[g_DATAWIDTH - 1 : 0] image_8[MAX_SIZE - 1 : 0];
  reg[DATA_BIT_WIDTH - 1 : 0] image_24[MAX_SIZE_24 - 1 : 0];
  integer i,
          j_rgb,
	  File_1;
  reg[32 : 0] counter_h_rgb,
              counter_v_rgb,
              counter_h_in_rgb,
              counter_v_in_rgb;
  reg[g_DATAWIDTH - 1 : 0] Yin;
  reg dataValidIn_sobel;
  wire[g_DATAWIDTH - 1 : 0] Yout;
  wire dataValidOut_sobel;
  reg[31 : 0] counter_h_sobel,
              counter_v_sobel,
              counter_h_in_sobel,
              counter_v_in_sobel;
  reg[g_DATAWIDTH - 1 : 0] Y;
  reg[g_DATAWIDTH - 1 : 0] Cb;
  reg[g_DATAWIDTH - 1 : 0] Cr;
  reg dataValidIn_y2r;
  wire[g_DATAWIDTH - 1 : 0] Red_out;
  wire[g_DATAWIDTH - 1 : 0] Green_out;
  wire[g_DATAWIDTH - 1 : 0] Blue_out;
  wire dataValidOut_y2r;
  reg[g_DATAWIDTH - 1 : 0] Mem_B[MAX_SIZE_24 - 2 : 0];
  reg[g_DATAWIDTH - 1 : 0] Mem_G[MAX_SIZE_24 - 2 : 0];
  reg[g_DATAWIDTH - 1 : 0] Mem_R[MAX_SIZE_24 - 2 : 0];
  reg[g_DATAWIDTH - 1 : 0] Mem_final[MAX_SIZE - 5 : 0];
  integer k_y2r,
          l,
          m;
  reg[31 : 0] counter_h_y2r,
              counter_v_y2r,
              counter_h_in_y2r,
              counter_v_in_y2r;
  initial
    begin
      $display("<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>");
      $display(">>>>>>>>            IMAGE EDGE DETECTION IP SIMULATION STARTED                      <<<<<<<");
      $display("<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>");
      Reset = 0;
      Clock = 0;
      H_sync_in = 0;
      V_sync_in = 0;
      H_active_in = 0;
      V_active_in = 0;
      Red = 0;
      Green = 0;
      Blue = 0;
      dataValidIn_rgb = 0;
      j_rgb = 1;
      counter_v_in_rgb = 0;
      counter_h_in_rgb = 0;
      counter_v_rgb = 0;
      counter_h_rgb = 0;
      Yin = 0;
      counter_v_sobel = 0;
      counter_h_sobel = 0;
      counter_v_in_sobel = 0;
      counter_h_in_sobel = 0;
      Y = 0;
      Cb = 0;
      Cr = 0;
      k_y2r = 0;
      l = 0;
      m = 0;
      counter_v_y2r = 0;
      counter_h_y2r = 0;
      counter_v_in_y2r = 0;
      counter_h_in_y2r = 0;
    end
  /************************************************************************
          Generates Clock signal
  *************************************************************************/
  initial
    begin
      forever
        #(CLKPERIOD / 2)
        Clock = ~Clock;
    end
  /************************************************************************
          RESET : Generates a Reset pulse 
  *************************************************************************/
  initial
    begin
      Reset = 1;
      #4
      Reset = 0;
      #2
      Reset = 1;
    end
  /************************************************************************
          Read the input image 
  *************************************************************************/
  initial
    begin
      $readmemh(IMAGE_FILE_NAME, image_8);
      for (i = 0; i < MAX_SIZE_24; i = i + 1)
        begin
          image_24[i] = ({ 16'b0, image_8[i * 3 + 3] } << 16) | ({ 16'b0, image_8[i * 3 + 2] } << 8) | (image_8[i * 3 + 1]);
        end
      $display("");
      $display("<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>");
      $display(">>>>>>>>                            IMAGE LOADED                                    <<<<<<<");
      $display("<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>");
    end
  /************************************************************************
  *********************RGB to YCbCr Conversion Starts**********************
  *************************************************************************/
  /************************************************************************
  Debug Signals: Output horizontal counter and output vertical counter
  *************************************************************************/
  initial
    begin
      forever
        @(posedge Clock)
        begin
          if (counter_h_rgb == (WIDTH - 1))
            begin
              counter_h_rgb = 0;
              counter_v_rgb = counter_v_rgb + 1;
            end
          else
            if (dataValidOut_rgb)
              counter_h_rgb = counter_h_rgb + 1;
        end
    end
  /************************************************************************
  Debug Signals: Input horizontal counter and input vertical counter
  *************************************************************************/
  initial
    begin
      forever
        @(counter_h_in_rgb)
        begin
          if (counter_h_in_rgb == (WIDTH))
            begin
              counter_v_in_rgb = counter_v_in_rgb + 1;
            end
          else
            counter_v_in_rgb = counter_v_in_rgb;
        end
    end
  /**************************************************************************
  Task to give the input (image) line after line with some delay between them
  ***************************************************************************/
  task read_image_delay_after_each_line_rgb;
    begin
      if (counter_v_in_rgb == (HEIGHT))
        begin
          dataValidIn_rgb = 0;
        end
      else
        begin
          if ((counter_h_in_rgb == (WIDTH)) && (Reset == 1))
            begin
              dataValidIn_rgb = 0;
              counter_h_in_rgb = 0;
              wait_for_next_line_rgb();
            end
          else
            begin
              if ((counter_h_in_rgb < (WIDTH)) && (Reset == 1))
                begin
                  dataValidIn_rgb = 1;
                  Red = image_24[j_rgb][7 : 0];
                  Green = image_24[j_rgb][15 : 8];
                  Blue = image_24[j_rgb][23 : 16];
                  j_rgb = j_rgb + 1;
                  counter_h_in_rgb = counter_h_in_rgb + 1;
                end
            end
        end
    end
  endtask
  /************************************************************************
  Task to define the amount of delay between two lines of input (image)
  *************************************************************************/
  task wait_for_next_line_rgb;
    begin
      repeat (WAIT)
        @(posedge Clock)
        ;
    end
  endtask
  initial
    begin
      forever
        @(posedge Clock)
        begin
          dataValidIn_sobel = dataValidOut_rgb;
        end
    end
  /************************************************************************
  *************************Display Enhancement Starts**********************
  *************************************************************************/
  /************************************************************************
  Debug Signals: Output horizontal counter and output vertical counter
  *************************************************************************/
  initial
    begin
      forever
        @(posedge Clock)
        begin
          if (counter_h_sobel == (WIDTH))
            begin
              counter_h_sobel = 0;
              counter_v_sobel = counter_v_sobel + 1;
            end
          else
            if (dataValidOut_sobel)
              counter_h_sobel = counter_h_sobel + 1;
        end
    end
  /************************************************************************
  Debug Signals: Input horizontal counter and input vertical counter
  *************************************************************************/
  initial
    begin
      forever
        @(counter_h_in_sobel)
        begin
          if (counter_h_in_sobel == (WIDTH))
            begin
              counter_v_in_sobel = counter_v_in_sobel + 1;
            end
          else
            counter_v_in_sobel = counter_v_in_sobel;
        end
    end
  /**************************************************************************
          Task to give YCbCr as input to Image Edge Detection IP
  ***************************************************************************/
  task read_image_delay_after_each_line_sobel;
    begin
      if ((counter_h_in_sobel == (WIDTH)) && (Reset == 1))
        counter_h_in_sobel = 0;
      else
        begin
          if ((counter_h_in_sobel < (WIDTH)) && (Reset == 1) && dataValidOut_rgb)
            begin
              Yin = Y_out;
              counter_h_in_sobel = counter_h_in_sobel + 1;
            end
        end
    end
  endtask
  initial
    begin
      forever
        @(posedge Clock)
        begin
          dataValidIn_y2r = dataValidOut_sobel;
        end
    end
  /************************************************************************
  *********************YCbCr to RGB Conversion Starts**********************
  *************************************************************************/
  /************************************************************************
  Debug Signals: Output horizontal counter and output vertical counter
  *************************************************************************/
  initial
    begin
      forever
        @(posedge Clock)
        begin
          if (counter_h_y2r == (WIDTH))
            begin
              counter_h_y2r = 0;
              counter_v_y2r = counter_v_y2r + 1;
            end
          else
            if (dataValidOut_y2r)
              counter_h_y2r = counter_h_y2r + 1;
        end
    end
  /************************************************************************
  Debug Signals: Input horizontal counter and input vertical counter
  *************************************************************************/
  initial
    begin
      forever
        @(counter_h_in_y2r)
        begin
          if (counter_h_in_y2r == (WIDTH))
            begin
              counter_v_in_y2r = counter_v_in_y2r + 1;
            end
          else
            counter_v_in_y2r = counter_v_in_y2r;
        end
    end
  /**************************************************************************
  Task to convert data in YCbCr space to RGB
  ***************************************************************************/
  task read_image_delay_after_each_line_y2r;
    begin
      if (counter_v_in_y2r == (HEIGHT))
        begin
          done_processing;
        end
      else
        begin
          if ((counter_h_in_y2r == (WIDTH)) && (Reset == 1))
            counter_h_in_y2r = 0;
          else
            begin
              if ((counter_h_in_y2r < (WIDTH)) && (Reset == 1) && dataValidOut_sobel)
                begin
                  Y = Yout;
                  Cb <= 'd128;
                  Cr <= 'd128;
                  counter_h_in_y2r = counter_h_in_y2r + 1;
                end
            end
        end
    end
  endtask
  /************************************************************************
  Task to check height of output image
  *************************************************************************/
  task done_processing;
    begin
      if (counter_v_y2r == HEIGHT)
        begin
          final_one;
        end
    end
  endtask
  task final_one;
    begin
      $display("");
      $display("<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>");
      $display(">>>>>>>>              SIMULATION COMPLETED                                          <<<<<<<");
      $display("<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>");
      $finish;
    end
  endtask
  /************************************************************************
  Process to write the output RGB into a memory
  *************************************************************************/
  initial
    begin
	     File_1 = $fopen("Edge_Detected_out.txt","wb");
      forever
        @(posedge Clock)
        begin
          if (dataValidOut_y2r)
            begin
	      $fwrite(File_1,"%x\n",Blue_out);
	      $fwrite(File_1,"%x\n",Green_out);
	      $fwrite(File_1,"%x\n",Red_out);
            end
        end
	$fclose(File_1);
    end
 
  /************************************************************************
  *************Process to initiate the testbench***************************
  *************************************************************************/
  initial
    begin
      $display("");
      $display("<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>");
      $display(">>>>>>>>                          PROCESSING OF IMAGE STARTED                       <<<<<<<");
      $display("<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>");
      forever
        @(posedge Clock)
        begin
         
          read_image_delay_after_each_line_rgb;
        end
    end
  initial
    begin
      forever
        @(posedge Clock)
        begin
          read_image_delay_after_each_line_sobel;
        end
    end
  initial
    begin
      forever
        @(posedge Clock)
        begin
          read_image_delay_after_each_line_y2r;
        end
    end
  /************************************************************************
  Component instantiation
  *************************************************************************/
  RGB2YCbCr #(.g_RGB_DATA_BIT_WIDTH(g_RGB_DATA_BIT_WIDTH), .g_YCbCr_DATA_BIT_WIDTH(g_YCbCr_DATA_BIT_WIDTH)) R2Y_Converter_0(.CLOCK_I(Clock),
                                                                                                                            .RESET_N_I(Reset),
                                                                                                                            .Red_i(Red),
                                                                                                                            .Green_i(Green),
                                                                                                                            .Blue_i(Blue),
                                                                                                                            .data_enable_i(dataValidIn_rgb),
                                                                                                                            .horz_sync_i(H_sync_in),
                                                                                                                            .vert_sync_i(V_sync_in),
                                                                                                                            .hactive_i(H_active_in),
                                                                                                                            .vactive_i(V_active_in),
                                                                                                                            .Y_out_o(Y_out),
                                                                                                                            .Cb_out_o(Cb_out),
                                                                                                                            .Cr_out_o(Cr_out),
                                                                                                                            .data_enable_o(dataValidOut_rgb),
                                                                                                                            .horz_sync_o(H_sync_out),
                                                                                                                            .vert_sync_o(V_sync_out),
                                                                                                                            .hactive_o(H_active_out),
                                                                                                                            .vactive_o(V_active_out));
  ImageEdgeDetection #(.g_DATAWIDTH(g_DATAWIDTH), .g_X_RES_WIDTH(g_X_RES_WIDTH), .g_DISPLAY_RESOLUTION(WIDTH), .g_VERT_DISPLAY_RESOLUTION(HEIGHT)) Edge_Detection_0(.SYS_CLK_I(Clock),
                                                                                                                                                                      .RESET_n_I(Reset),
                                                                                                                                                                      .Y_in_i(Yin),
                                                                                                                                                                      .Data_In_Vld_i(dataValidIn_sobel),
                                                                                                                                                                      .Y_Out_o(Yout),
                                                                                                                                                                      .data_Vld_out_o(dataValidOut_sobel));
  YCbCr2RGB #(.g_RGB_DATA_BIT_WIDTH(g_RGB_DATA_BIT_WIDTH), .g_YCbCr_DATA_BIT_WIDTH(g_YCbCr_DATA_BIT_WIDTH)) Y2R_Converter_0(.CLOCK_I(Clock),
                                                                                                                            .RESET_N_I(Reset),
                                                                                                                            .Y_i(Y),
                                                                                                                            .Cb_i(Cb),
                                                                                                                            .Cr_i(Cr),
                                                                                                                            .data_enable_i(dataValidIn_y2r),
                                                                                                                            .horz_sync_i(H_sync_in),
                                                                                                                            .vert_sync_i(V_sync_in),
                                                                                                                            .hactive_i(H_active_in),
                                                                                                                            .vactive_i(V_active_in),
                                                                                                                            .Red_out_o(Red_out),
                                                                                                                            .Green_out_o(Green_out),
                                                                                                                            .Blue_out_o(Blue_out),
                                                                                                                            .data_enable_o(dataValidOut_y2r),
                                                                                                                            .horz_sync_o(H_sync_out),
                                                                                                                            .vert_sync_o(V_sync_out),
                                                                                                                            .hactive_o(H_active_out),
                                                                                                                            .vactive_o(V_active_out));
endmodule
